#! /usr/bin/env python
from math import *
from adesklets.commands import *
import adesklets
from time import time
from random import randint
from statgrab import *
from os.path import join
import ConfigParser
import string
import re

class Monitor:
    def __init__(self, config, skin):
        self.config = {}
        self.convert_config( config )
        
        self.skin = skin
        self.delay =  float(self.config['delay'])
        self.last_update = 0
        self.s = (0,0)
        self.r = 0
        self.bg, self.bg_size = self.initiate_image(self.config['bg'])
        self.fg, self.fg_size = self.initiate_image(self.config['fg'])
        self.geom = ( 0, 0, self.bg_size[0], self.bg_size[1] )
        self.s = ( self.bg_size[0] / 2, self.bg_size[1] / 2 )
        self.s2 = ( self.bg_size[0], self.s[1] )
        self.r = self.s[0]
        #self.original_txt = self.config['txt_format']
        #self.text_coords = (int(self.config['txt_left']),\
                            #int(self.config['txt_top']))
        self.ifmaxval = self.is_maxval()
        self.old_dict = {}
        self.dict = {}
        self.update_dict()
        
        self.original_txt = []
        self.text_coords = []
        
        self.text = []
        self.txt_size = (0,0)
        self.get_orig_txt()
        #self.update_deg()
    
    def initiate_image( self, path ):
        path = join( self.skin, path )
        im = load_image( path )
        context_set_image( im )
        return (im, ( image_get_width() ,image_get_height() ))
    
    def get_coords( self, alpha, s, r ):
        x = s[0] + r * cos( alpha )
        y = s[1] + r * sin( alpha )
        return (x,y)
    
    def set_polygon( self, deg ):
        alpha = radians( deg )
        pt1 = self.get_coords( alpha, self.s, self.r )
        if deg <= 180:
            pt2 = (pt1[0], self.bg_size[1] )
            pt3 = ( 0, self.bg_size[1] )
            pt4 = ( 0, 0 )
            pt5 = ( self.bg_size[0], 0 )
            points = [ self.s2, self.s, pt1, pt2, pt3, pt4, pt5 ]
        else:
            pt2 = (pt1[0], 0 )
            pt3 = ( self.bg_size[0], 0 )
            points = [ self.s2, self.s, pt1, pt2, pt3 ]
        
        p = polygon_new()
        for pt in points:
            polygon_add_point( p, *pt )
        
        return p
    
    def create_buffer( self ):
        context_set_image( self.bg )
        bufbg = clone_image()
        context_set_image( self.fg )
        buffg = clone_image()
        context_set_image( buffg )
        context_set_color( 0,0,0,0 )
        
        polygon = self.set_polygon( self.deg )
        
        context_set_blend( False )
        image_fill_polygon( polygon )
        context_set_blend( True )
        
        polygon_free( polygon )
        
        context_set_image( bufbg )
        blend_image_onto_image( buffg, 1, *self.geom*2 )
        free_image( buffg )
        return bufbg
    
    def convert_config( self, config ):
        for k, v in config:
            self.config[k] = v
    
    def is_maxval( self ):
        if 'maxval' in self.config:
            return True
        return False
    
    def need_update( self ):
        """
        execute this method before update!
        """
        actual = time()
        var = actual - self.last_update
        if var < self.delay:
            return False
        
        self.last_update = actual
        self.update_dict()
        if self.dict == self.old_dict:
            return False
        self.old_dict = self.dict.copy()
        return True
    
    def update( self ):
        self.update_deg()
        self.update_text()
        buf = self.create_buffer()
        return buf
    
    def get_orig_txt( self ):
        counter = 1
        while True:
            txt = 'txt' + str(counter)
            if not txt in self.config:
                break
            txt = "self.config['" + txt
            self.original_txt.append( eval( txt + "']" ) )
            coords = ( (eval(txt + "_left']"), eval(txt+ "_top']")) )
            self.text_coords.append( (int(coords[0]), int(coords[1])) )
            counter += 1
    
    def update_text( self ):
        """
        gets text using a format given in skin config
        it substitutes tags ($cpuperc, $swapperc...)
        """
        #print 'dict', self.dict
        self.text = []
        for or_txt in self.original_txt:
            tpl = string.Template( or_txt )
            self.text.append( str(tpl.safe_substitute( self.dict )) )
    
    def update_dict( self ):
        """
        just overload it
        """
        self.dict = {}
        
    def update_deg( self ):
        """
        same as above
        """
        self.deg = 0



class Circle_meter(adesklets.Events_handler):
    def __init__(self, meter):
        self.cpu = meter
        adesklets.Events_handler.__init__(self)
        
    def __del__(self):
        adesklets.Events_handler.__del__(self)
    
    def get_color( self, color, alpha ):
        """
        color -> string in format: 'red,green,blue'
        alpha -> string, alpha channgel
        returnes 4 element tuple 
        """
        color = color.split(',',2)
        for num in range(0,3):
            color[num] = int( color[num] )
        color.append( int(alpha) )
        return color
    
    def setup_font( self ):
        """
        load font from skin config
        """
        adesklets.context_set_color( *self.font_color )
        adesklets.context_set_anti_alias( True )
        font = adesklets.load_font( self.font_str )
        adesklets.context_set_font( font )
        return font
    
    def ready( self ):
        """
        initialization
        """
        #self.cpu = SwapMeter( self.scfg, self.skin )
        #self.win_size = self.cpu.bg_size
        
        self.font_color = self.get_color( self.cpu.config['txt_color'],\
                                        self.cpu.config['txt_alpha'] )
        self.font_str = "%s/%s" % (self.cpu.config['txt_font'],\
                                    self.cpu.config['txt_fontsize'])
        self.font = self.setup_font()
        
        
        self.cpu.update_text()
        #print self.cpu.text
        self.win_size = self.what_is_text_size()
        self.win_geom = (0,0, self.win_size[0], self.win_size[1])
        #self.cpu.geom = (0, 0, self.win_size[0], self.win_size[1])
        
        #self.cpu.txt_size = get_text_size( 'My' )
           
        self.delay = mcfg.getint( 'Main', 'delay' )
        
        adesklets.window_resize(*self.win_size)
        adesklets.window_set_transparency(True)
        adesklets.context_set_blend( False )
        adesklets.window_show()
    
    def what_is_text_size( self ):
        lines_sizes = []
        for l in self.cpu.text:
            if len( l.strip() ) == 0:
                continue
            pat = re.compile( r'\W' )
            # substitute M for any non alphanumeric character
            # txt_size_func may not operate on such characters ;)
            l = pat.sub( 'M', l )
            #print l
            #print 'size:', get_text_size(l)
            lines_sizes.append( get_text_size(l) )
        ws = [ self.cpu.bg_size[0] ]
        hs = [ self.cpu.bg_size[1] ]
        for coords, sizes in zip( self.cpu.text_coords, lines_sizes ):
            ws.append( coords[0] + sizes[0] )
            hs.append( coords[1] + sizes[1] )
        return ( max(ws), max(hs) )
    
    def resize_window( self, size ):
        """
        checks if the new size is larger than the size before
        if it is... window is resized and a new size is returned
        """
        if size[0] > self.win_size[0] or size[1] > self.win_size[1]:
            print 'Resized!'
            self.win_size = size
            self.win_geom = ( 0, 0, size[0], size[1] )
            adesklets.window_resize( *size )
            return size
        return self.win_size
    
    def draw_text( self ):
        for line, coords in zip(self.cpu.text, self.cpu.text_coords):
            #print line
            #print coords
            if len( line.strip() ) != 0:
                adesklets.text_draw( coords[0], coords[1], line )
    
    def refresh( self ):
        if self.cpu.need_update():
            buf = self.cpu.update()
            #print 'refreshed'
        else:
            #print 'not refreshed'
            return
        self.resize_window( self.what_is_text_size() )
        newbuf = create_image( *self.win_size )
        context_set_color( *self.font_color )
        context_set_image( newbuf )
        context_set_blend( False )
        blend_image_onto_image( buf, 1, *self.cpu.geom*2 )
        self.draw_text()
        context_set_image( 0 )
        blend_image_onto_image( newbuf, 1, *self.win_geom*2 )
        free_image( newbuf )
        free_image( buf )
        #print images_info()
        
    def quit( self ):
        print 'So quickly? :('
        
    def alarm( self ):
        self.block()
        self.refresh()
        self.unblock()
        return self.delay


def get_config_obj(conffile ):
    fd = open( conffile, 'r' )
    cfg = ConfigParser.ConfigParser()
    cfg.readfp( fd )
    fd.close()
    return cfg

mainconffile = 'config'
skinconffile = 'config'

mcfg = get_config_obj( mainconffile )
skin = mcfg.get( 'Main', 'skindir' )
scfg = get_config_obj( join( skin, skinconffile ) )
