import string
import time
import datetime
from PIL import Image

class SongInfo:
    def __init__( self, ql_dict):
        """
        ql_dict -> dict returned by QuodLibet throught DBus interface
        """
        self.dict = ql_dict
        self.prepare_ql_dict()
        
        #self.start = datetime.datetime.fromtimestamp( ql_dict['~#laststarted'] )
        self.start = int(ql_dict['~#laststarted'])
        self.length = int(ql_dict['~#length'])
        self.get_started()
        self.dict['length'] = self.get_time_str( self.get_time_list( self.length ) )
        
        self.max_rating = 5
        self.rating = self.return_rating()
        self.dict['rating'] = self.rating
        self.update_dict()
    
    def prepare_ql_dict( self ):
        if not 'title' in self.dict:
            self.dict['title'] = 'Unknown'
        if not 'artist' in self.dict:
            self.dict['artist'] = 'Unknown'
        if not 'album' in self.dict:
            self.dict['album'] = 'Unknown'
    
    def return_rating( self ):
        return int( round( float(self.dict['~#rating']) * self.max_rating ) )
    
    def get_started( self ):
        # FIXME
        # when the same song is played twice this doesn't work
        act_time = time.time()
        if (act_time - self.start) > self.length:
            self.start = act_time
    
    def update_dict( self ):
        """
        updates needed values in dict
        """
        act_time = time.time()
        elap_s = int( round(act_time - self.start) )
        self.dict['t_elapsed'] = elap_s
        if elap_s > self.length:
            elap_s = self.length
        
        elapsed = self.get_time_str( self.get_time_list( elap_s ) )
        self.dict['elapsed'] = elapsed
        return True
    
    def get_time_list( self, secs ):
        """
        secs -> number of seconds
        it converts seconds to format hh:mm:ss
        """
        #secs = round( secs )
        minutes = 0
        hours = 0
        days = 0
        if secs >= 60:
            minutes = int(secs/60)
            secs = secs % 60
        if minutes >= 60:
            hours = int(minutes/60)
            minutes = minutes % 60
        if hours >= 24:
            days = int(hours/24)
            hours = hours % 24
        return { 'days': days,
                'hours': hours,
                'minutes': minutes,
                'seconds': secs }
    
    def get_time_str( self, time_list ):
        """
        time_list -> value returned by get_time_list method
        returns string of elapsed time
        """
        elapsed = "%s:%s" % (str(time_list['minutes']).zfill(2),\
                            str(time_list['seconds']).zfill(2))
        if time_list['hours'] > 0:
            elapsed = "%s:%s" % (str(time_list['hours']).zfill(2), elapsed)
        if time_list['days'] > 0:
            elapsed = "%s:%s" % (str(time_list['days']).zfill(2), elapsed)
        return elapsed
